@@ -9,11 +9,12 @@
 
 """ CSW request and response processor """
 
+import base64
 import warnings
 import StringIO
 import random
 from urllib import urlencode
-from urllib2 import urlopen
+from urllib2 import Request, urlopen
 
 from owslib.util import OrderedDict
 
@@ -39,7 +40,8 @@ schema_location = '%s %s' % (namespaces['csw'], schema)
 
 class CatalogueServiceWeb:
     """ csw request class """
-    def __init__(self, url, lang='en-US', version='2.0.2', timeout=10, skip_caps=False):
+    def __init__(self, url, lang='en-US', version='2.0.2', timeout=10, skip_caps=False,
+                 username=None, password=None):
         """
 
         Construct and process a GetCapabilities request
@@ -52,6 +54,8 @@ class CatalogueServiceWeb:
         - version: version (default is '2.0.2')
         - timeout: timeout in seconds
         - skip_caps: whether to skip GetCapabilities processing on init (default is False)
+        - username: username for HTTP basic authentication
+        - password: password for HTTP basic authentication
 
         """
 
@@ -59,6 +63,8 @@ class CatalogueServiceWeb:
         self.lang = lang
         self.version = version
         self.timeout = timeout
+        self.username = username
+        self.password = password
         self.service = 'CSW'
         self.exceptionreport = None
         self.owscommon = ows.OwsCommon('1.0.0')
@@ -256,7 +262,7 @@ class CatalogueServiceWeb:
 
         """
 
-        # construct request 
+        # construct request
         data = {
             'service': self.service,
             'version': self.version,
@@ -264,13 +270,13 @@ class CatalogueServiceWeb:
             'outputFormat': format,
             'outputSchema': outputschema,
             'elementsetname': esn,
-            'id': '',
+            'id': ','.join(id),
         }
 
-        self.request = '%s%s%s' % (bind_url(self.url), urlencode(data), ','.join(id))
+        self.request = '%s%s' % (bind_url(self.url), urlencode(data))
 
         self._invoke()
- 
+
         if self.exceptionreport is None:
             self.results = {}
             self.records = OrderedDict()
@@ -511,7 +517,7 @@ class CatalogueServiceWeb:
 
     def _parserecords(self, outputschema, esn):
         if outputschema == namespaces['gmd']: # iso 19139
-            for i in self._exml.findall('.//'+util.nspath_eval('gmd:MD_Metadata', namespaces)):
+            for i in self._exml.findall('.//'+util.nspath_eval('gmd:MD_Metadata', namespaces)) or self._exml.findall('.//'+util.nspath_eval('gmi:MI_Metadata', namespaces)):
                 val = i.find(util.nspath_eval('gmd:fileIdentifier/gco:CharacterString', namespaces))
                 identifier = self._setidentifierkey(util.testXMLValue(val))
                 self.records[identifier] = MD_Metadata(i)
@@ -584,21 +590,43 @@ class CatalogueServiceWeb:
         # do HTTP request
 
         if isinstance(self.request, basestring):  # GET KVP
-            self.response = urlopen(self.request, timeout=self.timeout).read()
+            req = Request(self.request)
+            if self.username is not None and self.password is not None:
+                base64string = base64.encodestring('%s:%s' % (self.username, self.password))[:-1]
+                req.add_header('Authorization', 'Basic %s' % base64string)
+            self.response = urlopen(req, timeout=self.timeout).read()
         else:
+            xml_post_url = self.url
+            # Get correct POST URL based on Operation list.
+            # If skip_caps=True, then self.operations has not been set, so use
+            # default URL.
+            if hasattr(self, 'operations'):
+                for op in self.operations:
+                    post_verbs = filter(lambda x: x.get('type').lower() == 'post', op.methods)
+                    if len(post_verbs) > 1:
+                        # Filter by constraints.  We must match a PostEncoding of "XML"
+                        try:
+                            xml_post_url = next(x for x in filter(list, ([pv.get('url') for const in pv.get('constraints') if const.name.lower() == "postencoding" and 'xml' in map(lambda x: x.lower(), const.values)] for pv in post_verbs)))[0]
+                        except StopIteration:
+                            # Well, just use the first one.
+                            xml_post_url = post_verbs[0].get('url')
+                    elif len(post_verbs) == 1:
+                        xml_post_url = post_verbs[0].get('url')
+
             self.request = cleanup_namespaces(self.request)
             # Add any namespaces used in the "typeNames" attribute of the
             # csw:Query element to the query's xml namespaces.
             for query in self.request.findall(util.nspath_eval('csw:Query', namespaces)):
                 ns = query.get("typeNames", None)
                 if ns is not None:
-                    # Pull out "gmd" from something like "gmd:MD_Metadata"
-                    ns = ns.split(":")[0]
-                    self.request = add_namespaces(self.request, ns)
+                    # Pull out "gmd" from something like "gmd:MD_Metadata" from the list
+                    # of typenames
+                    ns_keys = [x.split(':')[0] for x in ns.split(' ')]
+                    self.request = add_namespaces(self.request, ns_keys)
 
-            self.request = util.xml2string(etree.tostring(self.request))
+            self.request = util.element_to_string(self.request, encoding='utf-8')
 
-            self.response = util.http_post(self.url, self.request, self.lang, self.timeout)
+            self.response = util.http_post(xml_post_url, self.request, self.lang, self.timeout, self.username, self.password)
 
         # parse result see if it's XML
         self._exml = etree.parse(StringIO.StringIO(self.response))
