@@ -1,5 +1,6 @@
 import os
 from owslib.etree import etree
+from urlparse import urlparse
 
 def resource_file(filepath):
     return os.path.join(test_directory(), 'resources', filepath)
@@ -43,3 +44,15 @@ def compare_elements(a,b):
             return False
 
     return True
+
+def cast_tuple_int_list(tup):
+    """Set tuple float values to int for more predictable test results"""
+    return [int(a) for a in tup]
+
+def cast_tuple_int_list_srs(tup):
+    tup2 = cast_tuple_int_list(tup[:4])
+    tup2.append(tup[-1])
+    return tup2
+
+def sorted_url_query(url):
+    return sorted(urlparse(url).query.split("&"))
